@@ -18,16 +18,18 @@
 
 import os
 import errno
-import time
 import json
-from datetime import datetime, timedelta
-from gi.repository import GLib, GObject, Gtk, GdkPixbuf
+from datetime import timedelta
+from gi.repository import GLib, GObject, GdkPixbuf, Gtk
 from clocks import Clock
-from utils import Dirs, SystemSettings, LocalizedWeekdays, Alert
-from widgets import DigitalClockDrawing, SelectableIconView, ContentView
+from utils import Alert, Dirs, LocalizedWeekdays, SystemSettings, TimeString, WallClock
+from widgets import SelectableIconView, ContentView
 
 
-class AlarmsStorage():
+wallclock = WallClock.get_default()
+
+
+class AlarmsStorage:
     def __init__(self):
         self.filename = os.path.join(Dirs.get_user_data_dir(), "alarms.json")
 
@@ -36,8 +38,8 @@ class AlarmsStorage():
         for a in alarms:
             d = {
                 "name": a.name,
-                "hour": a.time.strftime("%H"),
-                "minute": a.time.strftime("%M"),
+                "hour": a.hour,
+                "minute": a.minute,
                 "days": a.days
             }
             alarm_list.append(d)
@@ -70,44 +72,39 @@ class AlarmsStorage():
 class AlarmItem:
     EVERY_DAY = [0, 1, 2, 3, 4, 5, 6]
 
-    def __init__(self, name=None, hour=None, minute=None, days=EVERY_DAY):
-        self.update(name=name, hour=hour, minute=minute, days=days)
-
-    def update(self, name=None, hour=None, minute=None, days=EVERY_DAY):
+    def __init__(self, name, hour, minute, days):
         self.name = name
         self.hour = hour
         self.minute = minute
-        self.days = days # list of numbers, 0 == Monday
+        self.days = days  # list of numbers, 0 == Monday
 
-        # an alarm without any day makes no sense...
-        if not self.days:
-            self.days = AlarmItem.EVERY_DAY
+        self._update_expiration_time()
+        self._reset_snooze(self.alarm_time)
 
-        self.time = None
-        if not hour == None and not minute == None:
-            self.update_expiration_time()
+        self.alarm_time_string = TimeString.format_time(self.alarm_time)
+        self.alarm_repeat_string = self._get_alarm_repeat_string()
+        self.is_light = self._get_is_light()
+        self.alert = Alert("alarm-clock-elapsed", name)
 
-    def update_expiration_time(self):
-        now = datetime.now()
-        dt = now.replace(hour=self.hour, minute=self.minute)
+    def _update_expiration_time(self):
+        now = wallclock.datetime
+        dt = now.replace(hour=self.hour, minute=self.minute, second=0, microsecond=0)
         # check if it can ring later today
         if dt.weekday() not in self.days or dt <= now:
             # otherwise if it can ring this week
-            next_days = [ d for d in self.days if d > dt.weekday() ]
+            next_days = [d for d in self.days if d > dt.weekday()]
             if next_days:
                 dt += timedelta(days=(next_days[0] - dt.weekday()))
             # otherwise next week
             else:
                 dt += timedelta(weeks=1, days=(self.days[0] - dt.weekday()))
-        self.time = dt
+        self.alarm_time = dt
 
-    def get_time_as_string(self):
-        if SystemSettings.get_clock_format() == "12h":
-            return self.time.strftime("%I:%M %p")
-        else:
-            return self.time.strftime("%H:%M")
+    def _reset_snooze(self, start_time):
+        self.snooze_time = start_time + timedelta(minutes=9)
+        self.is_snoozing = False
 
-    def get_alarm_repeat_string(self):
+    def _get_alarm_repeat_string(self):
         n = len(self.days)
         if n == 0:
             return ""
@@ -125,11 +122,29 @@ class AlarmItem:
                     days.append(LocalizedWeekdays.get_abbr(day_num))
             return ", ".join(days)
 
+    def _get_is_light(self):
+        return 7 <= self.hour <= 19
+
+    def snooze(self):
+        self.is_snoozing = True
+        self.alert.stop()
+
+    def stop(self):
+        self._reset_snooze(self.alarm_time)
+        self.alert.stop()
+
     def check_expired(self):
-        if datetime.now() > self.time:
-            self.update_expiration_time()
+        if wallclock.datetime > self.alarm_time:
+            self.alert.show()
+            self._reset_snooze(self.alarm_time)
+            self._update_expiration_time()
             return True
-        return False
+        elif self.is_snoozing and wallclock.datetime > self.snooze_time:
+            self.alert.show()
+            self._reset_snooze(self.snooze_time)
+            return True
+        else:
+            return False
 
 
 class AlarmDialog(Gtk.Dialog):
@@ -155,20 +170,14 @@ class AlarmDialog(Gtk.Dialog):
         content_area.pack_start(grid, True, True, 0)
 
         if alarm:
-            if self.cf == "12h":
-                h = int(alarm.time.strftime("%I"))
-                p = alarm.time.strftime("%p")
-            else:
-                h = alarm.hour
-                p = None
+            h = alarm.hour
             m = alarm.minute
             name = alarm.name
             days = alarm.days
         else:
-            t = time.localtime()
+            t = wallclock.localtime
             h = t.tm_hour
             m = t.tm_min
-            p = time.strftime("%p", t)
             name = _("New Alarm")
             days = []
 
@@ -201,11 +210,13 @@ class AlarmDialog(Gtk.Dialog):
             self.ampm = Gtk.ComboBoxText()
             self.ampm.append_text("AM")
             self.ampm.append_text("PM")
-            if p == "PM":
-                h = h - 12
-                self.ampm.set_active(1)
+            if h < 12:
+                self.ampm.set_active(0)  # AM
             else:
-                self.ampm.set_active(0)
+                self.ampm.set_active(1)  # PM
+                h -= 12
+            if h == 0:
+                h = 12
             grid.attach(self.ampm, 4, 0, 1, 1)
             self.hourselect.set_range(1.0, 12.0)
             self.hourselect.set_value(h)
@@ -260,171 +271,23 @@ class AlarmDialog(Gtk.Dialog):
         for btn in self.day_buttons:
             if btn.get_active():
                 days.append(btn.data)
+        # needed in case the first day of the week is not 0 (Monday)
+        days.sort()
+
+        # if no days were selected, create a daily alarm
+        if not days:
+            days = AlarmItem.EVERY_DAY
+
         alarm = AlarmItem(name, h, m, days)
         return alarm
 
 
-class AlarmWidget():
-    def __init__(self, view, alarm, alert):
-        self.view = view
-        self.alarm = alarm
-        self.alert = alert
-        timestr = alarm.get_time_as_string()
-        repeat = alarm.get_alarm_repeat_string()
-        self.drawing = DigitalClockDrawing()
-        is_light = self.get_is_light(int(timestr[:2]))
-        if is_light:
-            img = os.path.join(Dirs.get_image_dir(), "cities", "day.png")
-        else:
-            img = os.path.join(Dirs.get_image_dir(), "cities", "night.png")
-        self.drawing.render(timestr, img, is_light, repeat)
-        self.standalone = None
-
-    def get_is_light(self, hours):
-        return hours > 7 and hours < 19
-
-    def get_pixbuf(self):
-        return self.drawing.pixbuf
-
-    def get_alert(self):
-        return self.alert
-
-    def get_standalone_widget(self):
-        if not self.standalone:
-            self.standalone = StandaloneAlarm(self.view, self.alarm, self.alert)
-        return self.standalone
-
-
-class Alarm(Clock):
-    def __init__(self):
-        # Translators: "New" refers to an alarm
-        Clock.__init__(self, _("Alarm"), _("New"), True)
-
-        self.liststore = Gtk.ListStore(bool,
-                                       GdkPixbuf.Pixbuf,
-                                       str,
-                                       GObject.TYPE_PYOBJECT,
-                                       GObject.TYPE_PYOBJECT)
-
-        self.iconview = SelectableIconView(self.liststore, 0, 1, 2)
-
-        contentview = ContentView(self.iconview,
-                "alarm-symbolic",
-                _("Select <b>New</b> to add an alarm"))
-        self.add(contentview)
-
-        self.iconview.connect("item-activated", self._on_item_activated)
-        self.iconview.connect("selection-changed", self._on_selection_changed)
-
-        self.storage = AlarmsStorage()
-
-        self.load_alarms()
-        self.show_all()
-
-        self.timeout_id = GObject.timeout_add(1000, self._check_alarms)
-
-    def _check_alarms(self):
-        for i in self.liststore:
-            alarm = self.liststore.get_value(i.iter, 4)
-            if alarm.check_expired():
-                widget = self.liststore.get_value(i.iter, 3)
-                alert = widget.get_alert()
-                alert.show()
-                standalone = widget.get_standalone_widget()
-                standalone.set_ringing(True)
-                self.emit("show-standalone", widget)
-        return True
-
-    def _on_notification_activated(self, notif, action, data):
-        win = self.get_toplevel()
-        win.show_clock(self)
-
-    def _on_item_activated(self, iconview, path):
-        alarm = self.liststore[path][3]
-        self.emit("show-standalone", alarm)
-
-    def _on_selection_changed(self, iconview):
-        self.emit("selection-changed")
-
-    def set_selection_mode(self, active):
-        self.iconview.set_selection_mode(active)
-
-    @GObject.Property(type=bool, default=False)
-    def can_select(self):
-        return len(self.liststore) != 0
-
-    def get_selection(self):
-        return self.iconview.get_selection()
-
-    def delete_selected(self):
-        selection = self.get_selection()
-        alarms = []
-        for path in selection:
-            alarms.append(self.liststore[path][-1])
-        self.delete_alarms(alarms)
-
-    def load_alarms(self):
-        self.alarms = self.storage.load()
-        for alarm in self.alarms:
-            self.add_alarm_widget(alarm)
-
-    def add_alarm(self, alarm):
-        self.alarms.append(alarm)
-        self.storage.save(self.alarms)
-        self.add_alarm_widget(alarm)
-        self.show_all()
-
-    def add_alarm_widget(self, alarm):
-        alert = Alert("alarm-clock-elapsed", alarm.name,
-                      self._on_notification_activated)
-        widget = AlarmWidget(self, alarm, alert)
-        label = GLib.markup_escape_text(alarm.name)
-        view_iter = self.liststore.append([False,
-                                           widget.get_pixbuf(),
-                                           "<b>%s</b>" % label,
-                                           widget,
-                                           alarm])
-        self.notify("can-select")
-
-    def update_alarm(self, old_alarm, new_alarm):
-        i = self.alarms.index(old_alarm)
-        self.alarms[i] = new_alarm
-        self.storage.save(self.alarms)
-        self.iconview.unselect_all()
-        self.liststore.clear()
-        self.load_alarms()
-        self.notify("can-select")
-        return self.alarms[i]
-
-    def delete_alarms(self, alarms):
-        for a in alarms:
-            self.alarms.remove(a)
-        self.storage.save(self.alarms)
-        self.iconview.unselect_all()
-        self.liststore.clear()
-        self.load_alarms()
-        self.notify("can-select")
-
-    def open_new_dialog(self):
-        window = AlarmDialog(self.get_toplevel())
-        window.connect("response", self._on_dialog_response)
-        window.show_all()
-
-    def _on_dialog_response(self, dialog, response):
-        if response == 1:
-            alarm = dialog.get_alarm_item()
-            self.add_alarm(alarm)
-        dialog.destroy()
-
-
-class StandaloneAlarm(Gtk.EventBox):
-    def __init__(self, view, alarm, alert):
+class AlarmStandalone(Gtk.EventBox):
+    def __init__(self, view):
         Gtk.EventBox.__init__(self)
         self.get_style_context().add_class('view')
         self.get_style_context().add_class('content-view')
         self.view = view
-        self.alarm = alarm
-        self.alert = alert
         self.can_edit = True
 
         self.vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
@@ -476,33 +339,33 @@ class StandaloneAlarm(Gtk.EventBox):
         self.vbox.pack_start(hbox, False, False, 0)
         self.vbox.pack_start(Gtk.Label(), True, True, 0)
 
-        self.update()
+        self.set_alarm(None)
 
-        self.show_all()
-        self.set_ringing(False)
+    def set_alarm(self, alarm, ringing=False):
+        self.alarm = alarm
+        if alarm:
+            self.update()
+            self.show_all()
+            self.buttons.set_visible(ringing)
 
     def _on_stop_clicked(self, button):
-        self.alert.stop()
+        self.alarm.stop()
 
     def _on_snooze_clicked(self, button):
-        # Add 9 minutes, but without saving the change permanently
-        self.alarm.time += timedelta(minutes=9)
-        self.alert.stop()
+        self.alarm.snooze()
 
     def get_name(self):
         name = self.alarm.name
         return GLib.markup_escape_text(name)
 
-    def set_ringing(self, show):
-        self.buttons.set_visible(show)
-
     def update(self):
-        timestr = self.alarm.get_time_as_string()
-        repeat = self.alarm.get_alarm_repeat_string()
-        self.alarm_label.set_markup(
-            "<span size='72000' color='dimgray'><b>%s</b></span>" % timestr)
-        self.repeat_label.set_markup(
-            "<span size='large' color='dimgray'><b>%s</b></span>" % repeat)
+        if self.alarm:
+            timestr = self.alarm.alarm_time_string
+            repeat = self.alarm.alarm_repeat_string
+            self.alarm_label.set_markup(
+                "<span size='72000' color='dimgray'><b>%s</b></span>" % timestr)
+            self.repeat_label.set_markup(
+                "<span size='large' color='dimgray'><b>%s</b></span>" % repeat)
 
     def open_edit_dialog(self):
         window = AlarmDialog(self.get_toplevel(), self.alarm)
@@ -515,3 +378,135 @@ class StandaloneAlarm(Gtk.EventBox):
             self.alarm = self.view.update_alarm(self.alarm, new_alarm)
             self.update()
         dialog.destroy()
+
+
+class Alarm(Clock):
+    def __init__(self):
+        # Translators: "New" refers to an alarm
+        Clock.__init__(self, _("Alarm"), _("New"))
+
+        self.notebook = Gtk.Notebook()
+        self.notebook.set_show_tabs(False)
+        self.notebook.set_show_border(False)
+        self.add(self.notebook)
+
+        f = os.path.join(Dirs.get_image_dir(), "cities", "day.png")
+        self.daypixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(f, 160, 160)
+        f = os.path.join(Dirs.get_image_dir(), "cities", "night.png")
+        self.nightpixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(f, 160, 160)
+
+        self.liststore = Gtk.ListStore(bool, str, object)
+        self.iconview = SelectableIconView(self.liststore, 0, 1, self._thumb_data_func)
+        self.iconview.connect("item-activated", self._on_item_activated)
+        self.iconview.connect("selection-changed", self._on_selection_changed)
+
+        contentview = ContentView(self.iconview,
+                                  "alarm-symbolic",
+                                  _("Select <b>New</b> to add an alarm"))
+        self.notebook.append_page(contentview, None)
+
+        self.storage = AlarmsStorage()
+
+        self.load_alarms()
+        self.show_all()
+
+        self.standalone = AlarmStandalone(self)
+        self.notebook.append_page(self.standalone, None)
+
+        wallclock.connect("time-changed", self._check_alarms)
+
+    def _thumb_data_func(self, view, cell, store, i, data):
+        alarm = store.get_value(i, 2)
+        cell.text = alarm.alarm_time_string
+        cell.subtext = alarm.alarm_repeat_string
+        if alarm.is_light:
+            cell.props.pixbuf = self.daypixbuf
+            cell.css_class = "light"
+        else:
+            cell.props.pixbuf = self.nightpixbuf
+            cell.css_class = "dark"
+
+    def set_mode(self, mode):
+        self.mode = mode
+        if mode is Clock.Mode.NORMAL:
+            self.notebook.set_current_page(0)
+            self.iconview.set_selection_mode(False)
+        elif mode is Clock.Mode.STANDALONE:
+            self.notebook.set_current_page(1)
+        elif mode is Clock.Mode.SELECTION:
+            self.iconview.set_selection_mode(True)
+
+    @GObject.Signal
+    def alarm_ringing(self):
+        self.set_mode(Clock.Mode.STANDALONE)
+
+    def _check_alarms(self, *args):
+        for a in self.alarms:
+            if a.check_expired():
+                self.standalone.set_alarm(a, True)
+                self.emit("alarm-ringing")
+        return True
+
+    def _on_item_activated(self, iconview, path):
+        alarm = self.liststore[path][2]
+        self.standalone.set_alarm(alarm)
+        self.emit("item-activated")
+
+    def _on_selection_changed(self, iconview):
+        self.emit("selection-changed")
+
+    @GObject.Property(type=bool, default=False)
+    def can_select(self):
+        return len(self.liststore) != 0
+
+    def get_selection(self):
+        return self.iconview.get_selection()
+
+    def delete_selected(self):
+        selection = self.get_selection()
+        alarms = [self.liststore[path][2] for path in selection]
+        self.delete_alarms(alarms)
+        self.emit("selection-changed")
+
+    def load_alarms(self):
+        self.alarms = self.storage.load()
+        for alarm in self.alarms:
+            self._add_alarm_item(alarm)
+
+    def add_alarm(self, alarm):
+        self.alarms.append(alarm)
+        self.storage.save(self.alarms)
+        self._add_alarm_item(alarm)
+        self.show_all()
+
+    def _add_alarm_item(self, alarm):
+        label = GLib.markup_escape_text(alarm.name)
+        self.liststore.append([False, "<b>%s</b>" % label, alarm])
+        self.notify("can-select")
+
+    def update_alarm(self, old_alarm, new_alarm):
+        i = self.alarms.index(old_alarm)
+        self.alarms[i] = new_alarm
+        self.storage.save(self.alarms)
+        self.liststore.clear()
+        self.load_alarms()
+        self.notify("can-select")
+        return self.alarms[i]
+
+    def delete_alarms(self, alarms):
+        self.alarms = [a for a in self.alarms if a not in alarms]
+        self.storage.save(self.alarms)
+        self.liststore.clear()
+        self.load_alarms()
+        self.notify("can-select")
+
+    def open_new_dialog(self):
+        window = AlarmDialog(self.get_toplevel())
+        window.connect("response", self._on_dialog_response)
+        window.show_all()
+
+    def _on_dialog_response(self, dialog, response):
+        if response == 1:
+            alarm = dialog.get_alarm_item()
+            self.add_alarm(alarm)
+        dialog.destroy()
