@@ -19,7 +19,7 @@
 import os
 import sys
 from gettext import ngettext
-from gi.repository import Gtk, Gdk, GObject, GLib, Gio, GtkClutter
+from gi.repository import Gtk, Gdk, GObject, Gio, GtkClutter
 from clocks import Clock
 from world import World
 from alarm import Alarm
@@ -50,71 +50,84 @@ class Window(Gtk.ApplicationWindow):
                                                  "gtk-style.css"))
         context = Gtk.StyleContext()
         context.add_provider_for_screen(Gdk.Screen.get_default(),
-                                         css_provider,
-                                         Gtk.STYLE_PROVIDER_PRIORITY_USER)
+                                        css_provider,
+                                        Gtk.STYLE_PROVIDER_PRIORITY_USER)
 
         self.set_size_request(640, 480)
-        self.vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        self.embed = Embed(self.vbox)
-        self.add(self.embed)
-        self.notebook = Gtk.Notebook()
-        self.notebook.set_show_tabs(False)
-        self.notebook.set_show_border(False)
 
         self.world = World()
         self.alarm = Alarm()
         self.stopwatch = Stopwatch()
         self.timer = Timer()
-
+        self.world.connect("item-activated", self._on_item_activated)
+        self.world.connect("selection-changed", self._on_selection_changed)
+        self.alarm.connect("item-activated", self._on_item_activated)
+        self.alarm.connect("selection-changed", self._on_selection_changed)
+        self.alarm.connect("alarm-ringing", self._on_alarm_ringing)
+        self.timer.connect("alarm-ringing", self._on_alarm_ringing)
         self.views = (self.world, self.alarm, self.stopwatch, self.timer)
 
-        self.toolbar = ClocksToolbar(self.views, self.embed)
+        self.toolbar = ClocksToolbar(self.views)
+        self.toolbar.connect("back-clicked", self._on_back_clicked)
+        self.toolbar.connect("clock-changed", self._on_clock_changed)
+        self.toolbar.connect("done-clicked", self._on_done_clicked)
 
-        self.vbox.pack_start(self.toolbar, False, False, 0)
-
-        self.single_evbox = Gtk.EventBox()
-
-        self.vbox.pack_end(self.notebook, True, True, 0)
+        self.notebook = Gtk.Notebook()
+        self.notebook.set_show_tabs(False)
+        self.notebook.set_show_border(False)
         for view in self.views:
             self.notebook.append_page(view, None)
-        self.notebook.append_page(self.single_evbox, None)
 
-        self.world.connect("show-standalone", self._on_show_standalone)
-        self.alarm.connect("show-standalone", self._on_show_standalone)
+        self.embed = Embed(self.notebook)
+        self.embed._selectionToolbar._toolbarDelete.connect("clicked", self._on_delete_clicked)
 
-        self.toolbar.connect("view-clock", self._on_view_clock)
-        self.vbox.show_all()
-        self.show_all()
-        self.toolbar.show_overview_toolbar()
+        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        vbox.pack_start(self.toolbar, False, False, 0)
+        vbox.pack_end(self.embed, True, True, 0)
 
-    def show_clock(self, view):
-        self.toolbar.activate_view(view)
+        self.add(vbox)
 
-    def _on_show_standalone(self, widget, d):
-        def show_standalone_page():
-            widget = d.get_standalone_widget()
-            self.toolbar.show_standalone_toolbar(widget)
-            self.single_evbox.add(widget)
-            self.notebook.set_current_page(-1)
+        vbox.show()
 
-        if self.notebook.get_current_page() != len(self.views):
-            self.embed.spotlight(show_standalone_page)
+    def _on_item_activated(self, view):
+        def show_clock_standalone():
+            view.set_mode(Clock.Mode.STANDALONE)
+            self.toolbar.update_toolbar(view)
 
-    def _on_view_clock(self, button, view):
-        def show_clock_view():
-            for child in self.single_evbox.get_children():
-                self.single_evbox.remove(child)
-            view.unselect_all()
-            self.notebook.set_current_page(self.views.index(view))
-            self.toolbar.show_overview_toolbar()
+        self.embed.spotlight(show_clock_standalone)
 
-        if self.single_evbox.get_children():
-            self.embed.spotlight(show_clock_view)
-        else:
-            show_clock_view()
+    def _on_selection_changed(self, view):
+        if self.notebook.get_current_page() == self.views.index(view):
+            selection = view.get_selection()
+            n_selected = len(selection)
+            self.embed.set_show_selectionbar(n_selected > 0)
+            self.toolbar.update_toolbar(view)
+
+    def _on_alarm_ringing(self, view):
+        self.notebook.set_current_page(self.views.index(view))
+        self.toolbar.update_toolbar(view)
+
+    def _on_back_clicked(self, button, view):
+        def show_clock_overview():
+            view.set_mode(Clock.Mode.NORMAL)
+            self.toolbar.update_toolbar(view)
+
+        self.embed.spotlight(show_clock_overview)
+
+    def _on_clock_changed(self, button, view):
+        self.notebook.set_current_page(self.views.index(view))
+        self.toolbar.update_toolbar(view)
+
+    def _on_done_clicked(self, button, view):
+        self.embed.set_show_selectionbar(False)
+
+    def _on_delete_clicked(self, widget):
+        view = self.views[self.notebook.get_current_page()]
+        view.delete_selected()
 
     def _on_new_activated(self, action, param):
-        self.toolbar.current_view.open_new_dialog()
+        view = self.views[self.notebook.get_current_page()]
+        view.open_new_dialog()
 
     def _on_about_activated(self, action, param):
         about = Gtk.AboutDialog()
@@ -154,27 +167,44 @@ class Window(Gtk.ApplicationWindow):
 
 
 class ClockButton(Gtk.RadioButton):
-    _group = None
+    _radio_group = None
+    _size_group = None
 
     def __init__(self, text):
-        Gtk.RadioButton.__init__(self, group=ClockButton._group, draw_indicator=False)
-        self.text = text
+        Gtk.RadioButton.__init__(self, group=ClockButton._radio_group, draw_indicator=False)
+        if not ClockButton._radio_group:
+            ClockButton._radio_group = self
+        if not ClockButton._size_group:
+            ClockButton._size_group = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.HORIZONTAL)
+        # We use two labels to make sure they
+        # keep the same size even when using bold
         self.label = Gtk.Label()
         self.label.set_markup(text)
-        self.add(self.label)
+        self.bold_label = Gtk.Label()
+        self.bold_label.set_markup("<b>%s</b>" % text)
+        ClockButton._size_group.add_widget(self.label)
+        ClockButton._size_group.add_widget(self.bold_label)
+        if self.get_active():
+            self.add(self.bold_label)
+        else:
+            self.add(self.label)
         self.set_alignment(0.5, 0.5)
         self.set_size_request(100, 34)
         self.get_style_context().add_class('linked')
-        if not ClockButton._group:
-            ClockButton._group = self
 
     def do_toggled(self):
         try:
+            label = self.get_child()
+            self.remove(label)
+            # We need to unset the flag manually until GTK fixes
+            # https://bugzilla.gnome.org/show_bug.cgi?id=688519
+            label.unset_state_flags(Gtk.StateFlags.ACTIVE)
             if self.get_active():
-                self.label.set_markup("<b>%s</b>" % self.text)
+                self.add(self.bold_label)
             else:
-                self.label.set_markup("%s" % self.text)
-        except AttributeError:
+                self.add(self.label)
+            self.show_all()
+        except TypeError:
             # at construction the is no label yet
             pass
 
@@ -190,7 +220,7 @@ class SymbolicButton(Gtk.Button):
 
 
 class ClocksToolbar(Gtk.Toolbar):
-    def __init__(self, views, embed):
+    def __init__(self, views):
         Gtk.Toolbar.__init__(self)
 
         self.get_style_context().add_class("clocks-toolbar")
@@ -198,7 +228,6 @@ class ClocksToolbar(Gtk.Toolbar):
         self.get_style_context().add_class(Gtk.STYLE_CLASS_MENUBAR)
 
         self.views = views
-        self.embed = embed
 
         size_group = Gtk.SizeGroup(Gtk.SizeGroupMode.HORIZONTAL)
 
@@ -216,7 +245,7 @@ class ClocksToolbar(Gtk.Toolbar):
 
         self.back_button = SymbolicButton("go-previous-symbolic")
         self.back_button.connect("clicked",
-            lambda w: self.emit("view-clock", self.current_view))
+            lambda w: self.emit("back-clicked", self.current_view))
         left_box.pack_start(self.back_button, False, False, 0)
 
         center_item = Gtk.ToolItem()
@@ -239,7 +268,7 @@ class ClocksToolbar(Gtk.Toolbar):
             self.button_box.pack_start(button, True, True, 0)
             button.connect("toggled", self._on_toggled, view)
             self.view_buttons[view] = button
-            if view.has_selection_mode:
+            if hasattr(view, "can_select"):
                 view.connect("notify::can-select", self._on_can_select_changed)
             if view == views[0]:
                 self.current_view = view
@@ -270,104 +299,98 @@ class ClocksToolbar(Gtk.Toolbar):
         self.done_button = Gtk.Button(_("Done"))
         self.done_button.get_style_context().add_class('suggested-action')
         self.done_button.set_size_request(64, 34)
-        self.done_button.connect("clicked", self._on_done_clicked)
+        self.done_button.connect("clicked",
+            lambda w: self.emit("done-clicked", self.current_view))
         right_box.pack_end(self.done_button, False, False, 0)
 
-        self.selection_handler = 0
-
-        self.embed._selectionToolbar._toolbarDelete.connect("clicked", self._on_delete_clicked)
+        self.show_all()
+        self.update_toolbar(self.current_view)
 
     @GObject.Signal(arg_types=(Clock,))
-    def view_clock(self, view):
+    def back_clicked(self, view):
+        self.select_button.set_sensitive(view.can_select)
+
+    @GObject.Signal(arg_types=(Clock,))
+    def clock_changed(self, view):
         self.current_view = view
-        if view.has_selection_mode:
+        if hasattr(view, "can_select"):
             self.select_button.set_sensitive(view.can_select)
 
-    def activate_view(self, view):
+    @GObject.Signal(arg_types=(Clock,))
+    def done_clicked(self, view):
+        self.current_view.set_mode(Clock.Mode.NORMAL)
+        self.update_toolbar(self.current_view)
+
+    def update_toolbar(self, view):
         if view is not self.current_view:
             self.view_buttons[view].set_active(True)
+        if view.mode is Clock.Mode.NORMAL:
+            self._show_normal_toolbar()
+        elif view.mode is Clock.Mode.STANDALONE:
+            self._show_standalone_toolbar()
+        elif view.mode is Clock.Mode.SELECTION:
+            self._show_selection_toolbar()
 
-    def show_overview_toolbar(self):
+    def _show_normal_toolbar(self):
         self.get_style_context().remove_class("selection-mode")
-        self.standalone = None
         self.button_box.show()
         if self.current_view.new_label:
             self.new_button.set_label(self.current_view.new_label)
             self.new_button.show()
         else:
             self.new_button.hide()
-        self.select_button.set_visible(self.current_view.has_selection_mode)
+        self.select_button.set_visible(hasattr(self.current_view, "can_select"))
         self.back_button.hide()
         self.title_label.hide()
         self.edit_button.hide()
         self.done_button.hide()
-        if self.selection_handler:
-            self.current_view.disconnect_by_func(self._on_selection_changed)
-            self.selection_handler = 0
 
-    def show_standalone_toolbar(self, widget):
+    def _show_standalone_toolbar(self):
         self.get_style_context().remove_class("selection-mode")
-        self.standalone = widget
         self.button_box.hide()
         self.new_button.hide()
         self.select_button.hide()
         self.back_button.show_all()
-        self.title_label.set_markup("<b>%s</b>" % self.standalone.get_name())
+        standalone = self.current_view.standalone
+        self.title_label.set_markup("<b>%s</b>" % standalone.get_name())
         self.title_label.show()
-        self.edit_button.set_visible(self.standalone.can_edit)
+        self.edit_button.set_visible(standalone.can_edit)
         self.done_button.hide()
 
-    def show_selection_toolbar(self):
+    def _show_selection_toolbar(self):
         self.get_style_context().add_class("selection-mode")
-        self.standalone = None
         self.button_box.hide()
         self.new_button.hide()
         self.select_button.hide()
         self.back_button.hide()
-        self.set_selection_label(0)
+        self._set_selection_label(0)
         self.title_label.show()
         self.edit_button.hide()
         self.done_button.show()
-        self.selection_handler = \
-             self.current_view.connect("selection-changed",
-                                        self._on_selection_changed)
+        selection = self.current_view.get_selection()
+        self._set_selection_label(len(selection))
 
     def _on_toggled(self, widget, view):
-        self.emit("view-clock", view)
+        self.emit("clock-changed", view)
 
-    def set_selection_label(self, n):
+    def _set_selection_label(self, n):
         if n == 0:
             self.title_label.set_markup("(%s)" % _("Click on items to select them"))
         else:
             msg = ngettext("{0} item selected", "{0} items selected", n).format(n)
             self.title_label.set_markup("<b>%s</b>" % (msg))
 
-    def _on_selection_changed(self, view):
-        selection = view.get_selection()
-        n_selected = len(selection)
-        self.set_selection_label(n_selected)
-        self.embed.set_show_selectionbar(n_selected > 0)
-
     def _on_can_select_changed(self, view, pspec):
         if view == self.current_view:
             self.select_button.set_sensitive(view.can_select)
 
     def _on_select_clicked(self, button):
-        self.show_selection_toolbar()
-        self.current_view.set_selection_mode(True)
+        self.current_view.set_mode(Clock.Mode.SELECTION)
+        self.update_toolbar(self.current_view)
 
     def _on_edit_clicked(self, button):
-        self.standalone.open_edit_dialog()
-
-    def _on_done_clicked(self, widget):
-        self.show_overview_toolbar()
-        self.current_view.set_selection_mode(False)
-        self.embed.set_show_selectionbar(False)
-
-    def _on_delete_clicked(self, widget):
-        self.current_view.delete_selected()
-        self.set_selection_label(0)
-        self.embed.set_show_selectionbar(False)
+        standalone = self.current_view.standalone
+        standalone.open_edit_dialog()
 
 
 class ClocksApplication(Gtk.Application):
