@@ -17,7 +17,7 @@
 # Author: Seif Lotfy <seif.lotfy@collabora.co.uk>
 
 import time
-from gi.repository import Gtk, GObject
+from gi.repository import GLib, Gtk
 from clocks import Clock
 
 
@@ -81,15 +81,25 @@ class Stopwatch(Clock):
         self.right_button.connect("clicked", self._on_right_button_clicked)
 
         self.laps_store = Gtk.ListStore(str, str, str)
+
         cell = Gtk.CellRendererText()
+        cell.set_property("xalign", 1.0)
         n_column = Gtk.TreeViewColumn(_("Lap"), cell, markup=0)
-        n_column.set_expand(True)
+        n_column.set_expand(False)
+        n_column.set_property("alignment", 1.0)
+
         cell = Gtk.CellRendererText()
+        cell.set_property("xalign", 1.0)
         split_column = Gtk.TreeViewColumn(_("Split"), cell, markup=1)
         split_column.set_expand(True)
+        split_column.set_property("alignment", 1.0)
+
         cell = Gtk.CellRendererText()
+        cell.set_property("xalign", 1.0)
         tot_column = Gtk.TreeViewColumn(_("Total"), cell, markup=2)
         tot_column.set_expand(True)
+        tot_column.set_property("alignment", 1.0)
+
         self.laps_view = Gtk.TreeView(self.laps_store)
         self.laps_view.get_style_context().add_class("clocks-laps")
         self.laps_view.append_column(n_column)
@@ -98,10 +108,12 @@ class Stopwatch(Clock):
         scroll = Gtk.ScrolledWindow()
         scroll.get_style_context().add_class("clocks-laps-scroll")
         scroll.set_shadow_type(Gtk.ShadowType.IN)
-        scroll.set_vexpand(True);
+        scroll.set_vexpand(True)
         scroll.add(self.laps_view)
         vbox.pack_start(scroll, True, True, 0)
 
+        self.show_all()
+
     def _on_left_button_clicked(self, widget):
         if self.state in (Stopwatch.State.RESET, Stopwatch.State.STOPPED):
             self.state = Stopwatch.State.RUNNING
@@ -125,20 +137,19 @@ class Stopwatch(Clock):
             tot_h, tot_m, tot_s, split_h, split_m, split_s = self.get_time(True)
             n = "<span color='dimgray'> %d </span>" % (self.lap)
             if split_h > 0:
-                s = "<span size ='larger'>%i:%02i:%04.2f</span>" % (split_h, split_m, split_s)
+                s = "<span size ='larger'>%i:%02i:%05.2f</span>" % (split_h, split_m, split_s)
             else:
-                s = "<span size ='larger'>%02i:%04.2f</span>" % (split_m, split_s)
+                s = "<span size ='larger'>%02i:%05.2f</span>" % (split_m, split_s)
             if tot_h:
-                t = "<span size ='larger'>%i:%02i:%04.2f</span>" % (tot_h, tot_m, tot_s)
+                t = "<span size ='larger'>%i:%02i:%05.2f</span>" % (tot_h, tot_m, tot_s)
             else:
-                t = "<span size ='larger'>%02i:%04.2f</span>" % (tot_m, tot_s)
+                t = "<span size ='larger'>%02i:%05.2f</span>" % (tot_m, tot_s)
             i = self.laps_store.append([n, s, t])
             p = self.laps_store.get_path(i)
             self.laps_view.scroll_to_cell(p, None, False, 0, 0)
         if self.state == Stopwatch.State.STOPPED:
             self.state = Stopwatch.State.RESET
-            self.time_diff = 0
-            self.lap = 0
+            self.reset()
             self.left_label.set_markup(Stopwatch.BUTTON_MARKUP % (_("Start")))
             self.left_button.get_style_context().add_class("clocks-go")
             self.right_button.set_sensitive(False)
@@ -165,23 +176,30 @@ class Stopwatch(Clock):
         else:
             self.time_label.set_markup(Stopwatch.LABEL_MARKUP % (m, s))
 
+    def _add_timeout(self):
+        if self.timeout_id == 0:
+            self.timeout_id = GLib.timeout_add(100, self.count)
+
+    def _remove_timeout(self):
+        if self.timeout_id != 0:
+            GLib.source_remove(self.timeout_id)
+            self.timeout_id = 0
+
     def start(self):
         if self.timeout_id == 0:
             curr = time.time()
             self.start_time = curr
             self.lap_start_time = curr
-            self.timeout_id = GObject.timeout_add(100, self.count)
+            self._add_timeout()
 
     def stop(self):
-        GObject.source_remove(self.timeout_id)
-        self.timeout_id = 0
         curr = time.time()
+        self._remove_timeout()
         self.time_diff = self.time_diff + (curr - self.start_time)
         self.lap_time_diff = self.lap_time_diff + (curr - self.lap_start_time)
 
     def reset(self):
-        GObject.source_remove(self.timeout_id)
-        self.timeout_id = 0
+        self.lap = 0
         self.time_diff = 0
         self.lap_time_diff = 0
 
@@ -189,3 +207,12 @@ class Stopwatch(Clock):
         (h, m, s) = self.get_time()
         self.set_time_label(h, m, s)
         return True
+
+    def _ui_freeze(self, widget):
+        if self.state == Stopwatch.State.RUNNING:
+            self._remove_timeout()
+
+    def _ui_thaw(self, widget):
+        if self.state == Stopwatch.State.RUNNING:
+            self.count()
+            self._add_timeout()
