@@ -17,7 +17,8 @@
 # Author: Seif Lotfy <seif.lotfy@collabora.co.uk>
 
 import time
-from gi.repository import GObject, Gtk
+import math
+from gi.repository import GLib,  GObject, Gtk
 from clocks import Clock
 from utils import Alert
 from widgets import Spinner
@@ -155,6 +156,8 @@ class Timer(Clock):
         Clock.__init__(self, _("Timer"))
         self.state = Timer.State.STOPPED
         self.timeout_id = 0
+        self._last_set_time = None
+        self._ui_is_frozen = False
 
         self.notebook = Gtk.Notebook()
         self.notebook.set_show_tabs(False)
@@ -162,7 +165,7 @@ class Timer(Clock):
         self.add(self.notebook)
 
         # force the time label and the spinner to the same size
-        size_group = Gtk.SizeGroup(Gtk.SizeGroupMode.VERTICAL);
+        size_group = Gtk.SizeGroup(Gtk.SizeGroupMode.VERTICAL)
 
         self.setup_screen = TimerSetupScreen(self, size_group)
         self.notebook.append_page(self.setup_screen, None)
@@ -170,12 +173,13 @@ class Timer(Clock):
         self.timer_screen = TimerScreen(self, size_group)
         self.notebook.append_page(self.timer_screen, None)
 
-        self.alert = Alert("complete", "Ta Da !",
-                           self._on_notification_activated)
+        self.show_all()
 
-    def _on_notification_activated(self, notif, action, data):
-        win = self.get_toplevel()
-        win.show_clock(self)
+        self.alert = Alert("complete", "Ta Da !")
+
+    @GObject.Signal
+    def alarm_ringing(self):
+        self.alert.show()
 
     def show_setup_screen(self, reset):
         self.notebook.set_current_page(0)
@@ -186,12 +190,13 @@ class Timer(Clock):
         self.notebook.set_current_page(1)
 
     def _add_timeout(self):
-        self.timeout_id = GObject.timeout_add(250, self.count)
+        if self.timeout_id == 0:
+            self.timeout_id = GLib.timeout_add(250, self.count)
 
     def _remove_timeout(self):
         if self.timeout_id != 0:
-            GObject.source_remove(self.timeout_id)
-        self.timeout_id = 0
+            GLib.source_remove(self.timeout_id)
+            self.timeout_id = 0
 
     def start(self):
         if self.state == Timer.State.STOPPED and self.timeout_id == 0:
@@ -221,15 +226,28 @@ class Timer(Clock):
     def count(self):
         t = time.time()
         if t >= self.deadline:
-            self.alert.show()
+            self.emit("alarm-ringing")
             self.state = Timer.State.STOPPED
             self._remove_timeout()
             self.timer_screen.set_time(0, 0, 0)
             self.show_setup_screen(False)
             return False
-        else:
-            r = self.deadline - t
-            m, s = divmod(r, 60)
+        elif not self._ui_is_frozen:
+            # math.ceil() is needed because we count backwards. It assures the
+            # display shows the past and not the future, e.g. show 1 s and not
+            # 0 s when we are at 0.3 s.
+            display_time = math.ceil(self.deadline - t)
+            m, s = divmod(display_time, 60)
             h, m = divmod(m, 60)
-            self.timer_screen.set_time(h, m, s)
-            return True
+            if (h, m, s) != self._last_set_time:
+                self.timer_screen.set_time(h, m, s)
+                self._last_set_time = (h, m, s)
+        return True
+
+    def _ui_freeze(self, widget):
+        self._ui_is_frozen = True
+
+    def _ui_thaw(self, widget):
+        self._ui_is_frozen = False
+        if self.state == Timer.State.RUNNING:
+            self.count()
