@@ -20,19 +20,20 @@ import os
 import errno
 import time
 import json
-from gi.repository import GLib, GObject, Gio, Gtk, GdkPixbuf
+from gi.repository import GLib, GObject, Gio, GdkPixbuf, Gtk
 from gi.repository import GWeather
 from clocks import Clock
-from utils import Dirs, SystemSettings
-from widgets import DigitalClockDrawing, SelectableIconView, ContentView
+from utils import Dirs, TimeString, WallClock
+from widgets import SelectableIconView, ContentView
 
 
 # keep the GWeather world around as a singletom, otherwise
 # if is garbage collected get_city_name etc fail.
 gweather_world = GWeather.Location.new_world(True)
+wallclock = WallClock.get_default()
 
 
-class WorldClockStorage():
+class WorldClockStorage:
     def __init__(self):
         self.filename = os.path.join(Dirs.get_user_data_dir(), "clocks.json")
 
@@ -51,7 +52,7 @@ class WorldClockStorage():
             for l in location_codes:
                 location = GWeather.Location.find_by_station_code(gweather_world, l)
                 if location:
-                    clock = DigitalClock(location)
+                    clock = ClockItem(location)
                     clocks.append(clock)
         except IOError as e:
             if e.errno == errno.ENOENT:
@@ -115,261 +116,112 @@ class NewWorldClockDialog(Gtk.Dialog):
 
     def _icon_released(self, icon_pos, event, data):
         if self.entry.get_icon_gicon(
-            Gtk.EntryIconPosition.SECONDARY) == self.clear_gicon:
+                Gtk.EntryIconPosition.SECONDARY) == self.clear_gicon:
             self.entry.set_text('')
             self.entry.set_icon_from_gicon(
-              Gtk.EntryIconPosition.SECONDARY, self.find_gicon)
+                Gtk.EntryIconPosition.SECONDARY, self.find_gicon)
             self.set_response_sensitive(1, False)
 
 
-class DigitalClock():
+class ClockItem:
     def __init__(self, location):
         self.location = location
-        self._last_sunrise = time.strptime("197007:00", "%Y%H:%M")
-        self.sunrise = self._last_sunrise
-        self._last_sunset = time.strptime("197019:00", "%Y%H:%M")
-        self.sunset = self._last_sunset
-        self.get_sunrise_sunset()
-
-        self.path = None
-        self.list_store = None
+        self.name = self._get_location_name()
 
         weather_timezone = self.location.get_timezone()
         timezone = GLib.TimeZone.new(weather_timezone.get_tzid())
-        i = timezone.find_interval(GLib.TimeType.UNIVERSAL, time.time())
+        i = timezone.find_interval(GLib.TimeType.UNIVERSAL, wallclock.time)
         location_offset = timezone.get_offset(i)
 
         timezone = GLib.TimeZone.new_local()
-        i = timezone.find_interval(GLib.TimeType.UNIVERSAL, time.time())
+        i = timezone.find_interval(GLib.TimeType.UNIVERSAL, wallclock.time)
         here_offset = timezone.get_offset(i)
 
         self.offset = location_offset - here_offset
 
-        self._last_time = None
-        self.drawing = DigitalClockDrawing()
-        self.standalone = None
-        self.update()
+        self.sunrise = None
+        self.sunset = None
+        self.sunrise_string = None
+        self.sunset_string = None
 
-    def get_location_time(self, secs=None):
+        self._update_sunrise_sunset()
+
+        self.update_time()
+
+    def _get_location_name(self):
+        nation = self.location
+        while nation and nation.get_level() != GWeather.LocationLevel.COUNTRY:
+            nation = nation.get_parent()
+        if nation:
+            return self.location.get_city_name() + ', ' + nation.get_name()
+        else:
+            return self.location.get_city_name()
+
+    def _get_location_time(self, secs=None):
         if not secs:
-            secs = time.time()
+            secs = wallclock.time
         t = secs + self.offset
         t = time.localtime(t)
         return t
 
-    def update(self):
-        clock_format = SystemSettings.get_clock_format()
-        location_time = self.get_location_time()
-        if clock_format == '12h':
-            t = time.strftime("%I:%M %p", location_time)
-        else:
-            t = time.strftime("%H:%M", location_time)
-        if t.startswith("0"):
-            t = t[1:]
-        if not t == self._last_time \
-                or not self.sunrise == self._last_sunrise \
-                or not self.sunset == self._last_sunset:
-            is_light = self.get_is_light(location_time)
-            if is_light:
-                img = os.path.join(Dirs.get_image_dir(), "cities", "day.png")
-            else:
-                img = os.path.join(Dirs.get_image_dir(), "cities", "night.png")
-            day = self.get_day()
-            if day == "Today":
-                self.drawing.render(t, img, is_light)
-            else:
-                self.drawing.render(t, img, is_light, day)
-            if self.path and self.list_store:
-                self.list_store[self.path][1] = self.drawing.pixbuf
-            if self.standalone:
-                self.standalone.update(img, t, self.sunrise, self.sunset)
+    def _get_day_string(self):
+        clock_time_day = self.location_time.tm_yday
+        local_time_day = wallclock.localtime.tm_yday
 
-        self._last_time = t
+        # if its 31st Dec here and 1st Jan there, clock_time_day = 1,
+        # local_time_day = 365/366
+        # if its 1st Jan here and 31st Dec there, clock_time_day = 365/366,
+        # local_time_day = 1
+        if clock_time_day > local_time_day:
+            if local_time_day == 1:
+                return _("Yesterday")
+            else:
+                return _("Tomorrow")
+        elif clock_time_day < local_time_day:
+            if clock_time_day == 1:
+                return _("Tomorrow")
+            else:
+                return _("Yesterday")
 
-    def get_sunrise_sunset(self):
+    def _update_sunrise_sunset(self):
         self.weather = GWeather.Info(location=self.location, world=gweather_world)
         self.weather.connect('updated', self._on_weather_updated)
         self.weather.update()
 
     def _on_weather_updated(self, weather):
         # returned as the time here
-        ok, sunrise = weather.get_value_sunrise()
-        ok, sunset = weather.get_value_sunset()
-        self._last_sunrise = self.sunrise
-        self._last_sunset = self.sunset
-        self.sunrise = self.get_location_time(sunrise)
-        self.sunset = self.get_location_time(sunset)
-        self.update()
+        ok1, sunrise = weather.get_value_sunrise()
+        ok2, sunset = weather.get_value_sunset()
+        if ok1 and ok2:
+            self.sunrise = self._get_location_time(sunrise)
+            self.sunset = self._get_location_time(sunset)
+            self.sunrise_string = TimeString.format_time(self.sunrise)
+            self.sunset_string = TimeString.format_time(self.sunset)
+            self.is_light = self._get_is_light()
 
-    def get_pixbuf(self):
-        return self.drawing.pixbuf
+    def _get_is_light(self):
+        current = self.location_time
+        if self.sunrise:
+            return self.sunrise <= current <= self.sunset
+        else:
+            # default fallback when we have no sunrise/sunset times,
+            # as we only have images for either day or night
+            return 7 <= current.tm_hour <= 19
 
-    def get_standalone_widget(self):
-        if not self.standalone:
-            self.standalone = StandaloneClock(self.location, self.sunrise, self.sunset)
-        self.update()
-        return self.standalone
-
-    def get_day(self):
-        clock_time_day = self.get_location_time().tm_yday
-        local_time_day = time.localtime().tm_yday
-
-        if clock_time_day == local_time_day:
-            return "Today"
-        # if its 31st Dec here and 1st Jan there, clock_time_day = 1,
-        # local_time_day = 365/366
-        # if its 1st Jan here and 31st Dec there, clock_time_day = 365/366,
-        # local_time_day = 1
-        elif clock_time_day > local_time_day:
-            if local_time_day == 1:
-                return "Yesterday"
-            else:
-                return "Tomorrow"
-        elif clock_time_day < local_time_day:
-            if clock_time_day == 1:
-                return "Tomorrow"
-            else:
-                return "Yesterday"
-
-    def get_is_light(self, current):
-        if current.tm_hour < self.sunrise.tm_hour \
-                or current.tm_hour > self.sunset.tm_hour:
-            return False
-        elif current.tm_hour > self.sunrise.tm_hour \
-                and current.tm_hour < self.sunset.tm_hour:
-            return True
-        elif current.tm_hour == self.sunrise.tm_hour:
-            if current.tm_min >= self.sunrise.tm_min:
-                return True
-            else:
-                return False
-        elif current.tm_hour == self.sunset.tm_hour:
-            if current.tm_min <= self.sunrise.tm_min:
-                return True
-            else:
-                return False
-
-    def set_path(self, list_store, path):
-        self.path = path
-        self.list_store = list_store
+    def update_time(self):
+        self.location_time = self._get_location_time()
+        self.time_string = TimeString.format_time(self.location_time)
+        self.day_string = self._get_day_string()
+        self.is_light = self._get_is_light()
 
 
-class World(Clock):
+class ClockStandalone(Gtk.EventBox):
     def __init__(self):
-        # Translators: "New" refers to a world clock
-        Clock.__init__(self, _("World"), _("New"), True)
-
-        self.liststore = Gtk.ListStore(bool,
-                                       GdkPixbuf.Pixbuf,
-                                       str,
-                                       GObject.TYPE_PYOBJECT)
-
-        self.iconview = SelectableIconView(self.liststore, 0, 1, 2)
-
-        contentview = ContentView(self.iconview,
-                "document-open-recent-symbolic",
-                 _("Select <b>New</b> to add a world clock"))
-        self.add(contentview)
-
-        self.iconview.connect("item-activated", self._on_item_activated)
-        self.iconview.connect("selection-changed", self._on_selection_changed)
-
-        self.storage = WorldClockStorage()
-        self.clocks = []
-        self.load_clocks()
-        self.show_all()
-
-        self.timeout_id = GObject.timeout_add(1000, self._update_clocks)
-
-    def _update_clocks(self):
-        for c in self.clocks:
-            c.update()
-        return True
-
-    def _on_item_activated(self, iconview, path):
-        d = self.liststore[path][3]
-        self.emit("show-standalone", d)
-
-    def _on_selection_changed(self, iconview):
-        self.emit("selection-changed")
-
-    def set_selection_mode(self, active):
-        self.iconview.set_selection_mode(active)
-
-    @GObject.Property(type=bool, default=False)
-    def can_select(self):
-        return len(self.liststore) != 0
-
-    def get_selection(self):
-        return self.iconview.get_selection()
-
-    def delete_selected(self):
-        selection = self.get_selection()
-        items = []
-        for treepath in selection:
-            items.append(self.liststore[treepath][3])
-        self.delete_clocks(items)
-
-    def load_clocks(self):
-        self.clocks = self.storage.load()
-        for clock in self.clocks:
-            self.add_clock_widget(clock)
-
-    def add_clock(self, location):
-        for c in self.clocks:
-            if c.location.get_code() == location.get_code():
-                # duplicate
-                return
-        clock = DigitalClock(location)
-        self.clocks.append(clock)
-        self.storage.save(self.clocks)
-        self.add_clock_widget(clock)
-        self.show_all()
-
-    def add_clock_widget(self, clock):
-        name = clock.location.get_city_name()
-        label = GLib.markup_escape_text(name)
-        view_iter = self.liststore.append([False,
-                                           clock.get_pixbuf(),
-                                           "<b>%s</b>" % label,
-                                           clock])
-        path = self.liststore.get_path(view_iter)
-        clock.set_path(self.liststore, path)
-        self.notify("can-select")
-
-    def delete_clocks(self, clocks):
-        for c in clocks:
-            self.clocks.remove(c)
-        self.storage.save(self.clocks)
-        self.iconview.unselect_all()
-        self.liststore.clear()
-        self.load_clocks()
-        self.notify("can-select")
-
-    def open_new_dialog(self):
-        window = NewWorldClockDialog(self.get_toplevel())
-        window.connect("response", self._on_dialog_response)
-        window.show_all()
-
-    def _on_dialog_response(self, dialog, response):
-        if response == 1:
-            l = dialog.get_location()
-            self.add_clock(l)
-        dialog.destroy()
-
-
-class StandaloneClock(Gtk.EventBox):
-    def __init__(self, location, sunrise, sunset):
         Gtk.EventBox.__init__(self)
         self.get_style_context().add_class('view')
         self.get_style_context().add_class('content-view')
-        self.location = location
         self.can_edit = False
         self.time_label = Gtk.Label()
-        self.sunrise = sunrise
-        self.sunset = sunset
-
-        self.clock_format = None
 
         self.vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self.add(self.vbox)
@@ -409,44 +261,164 @@ class StandaloneClock(Gtk.EventBox):
         sunset_hbox.pack_start(sunset_label, False, False, 0)
         sunset_hbox.pack_start(self.sunset_time_label, False, False, 0)
 
-        sunbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        sunbox.set_homogeneous(True)
-        sunbox.set_spacing(3)
-        sunbox.pack_start(sunrise_hbox, False, False, 3)
-        sunbox.pack_start(sunset_hbox, False, False, 3)
+        self.sunbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        self.sunbox.set_homogeneous(True)
+        self.sunbox.set_spacing(3)
+        self.sunbox.pack_start(sunrise_hbox, False, False, 3)
+        self.sunbox.pack_start(sunset_hbox, False, False, 3)
 
         hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
         hbox.pack_start(Gtk.Label(), True, True, 0)
-        hbox.pack_start(sunbox, False, False, 0)
+        hbox.pack_start(self.sunbox, False, False, 0)
         hbox.pack_start(Gtk.Label(), True, True, 0)
         self.vbox.pack_end(hbox, False, False, 30)
 
-        self.show_all()
+        self.set_clock(None)
+
+    def set_clock(self, clock):
+        self.clock = clock
+        if clock:
+            self.update()
+            self.show_all()
+            if not clock.sunrise:
+                self.sunbox.hide()
 
     def get_name(self):
-        return GLib.markup_escape_text(self.location.get_city_name())
+        return GLib.markup_escape_text(self.clock.name)
 
-    def update(self, img, text, sunrise, sunset):
-        size = 72000  # FIXME: (self.get_allocation().height / 300) * 72000
-        self.time_label.set_markup(
-            "<span size='%i' color='dimgray'><b>%s</b></span>" % (size, text))
-        clock_format = SystemSettings.get_clock_format()
-        if clock_format != self.clock_format or \
-                sunrise != self.sunrise or sunset != self.sunset:
-            self.sunrise = sunrise
-            self.sunset = sunset
-            if clock_format == "12h":
-                sunrise_str = time.strftime("%I:%M %p", sunrise)
-                sunset_str = time.strftime("%I:%M %p", sunset)
+    def update(self):
+        if self.clock:
+            timestr = self.clock.time_string
+            self.time_label.set_markup(
+                "<span size='72000' color='dimgray'><b>%s</b></span>" % timestr)
+            if self.clock.sunrise_string and self.clock.sunset_string:
+                self.sunrise_time_label.set_markup(
+                    "<span size ='large'>%s</span>" % self.clock.sunrise_string)
+                self.sunset_time_label.set_markup(
+                    "<span size ='large'>%s</span>" % self.clock.sunset_string)
+                self.sunbox.show()
             else:
-                sunrise_str = time.strftime("%H:%M", sunrise)
-                sunset_str = time.strftime("%H:%M", sunset)
-            if sunrise_str.startswith("0"):
-                sunrise_str = sunrise_str[1:]
-            if sunset_str.startswith("0"):
-                sunset_str = sunset_str[1:]
-            self.sunrise_time_label.set_markup(
-                "<span size ='large'>%s</span>" % sunrise_str)
-            self.sunset_time_label.set_markup(
-                "<span size ='large'>%s</span>" % sunset_str)
-        self.clock_format = clock_format
+                self.sunbox.hide()
+
+
+class World(Clock):
+    def __init__(self):
+        # Translators: "New" refers to a world clock
+        Clock.__init__(self, _("World"), _("New"))
+
+        self.notebook = Gtk.Notebook()
+        self.notebook.set_show_tabs(False)
+        self.notebook.set_show_border(False)
+        self.add(self.notebook)
+
+        f = os.path.join(Dirs.get_image_dir(), "cities", "day.png")
+        self.daypixbuf = GdkPixbuf.Pixbuf.new_from_file(f)
+        f = os.path.join(Dirs.get_image_dir(), "cities", "night.png")
+        self.nightpixbuf = GdkPixbuf.Pixbuf.new_from_file(f)
+
+        self.liststore = Gtk.ListStore(bool, str, object)
+        self.iconview = SelectableIconView(self.liststore, 0, 1, self._thumb_data_func)
+        self.iconview.connect("item-activated", self._on_item_activated)
+        self.iconview.connect("selection-changed", self._on_selection_changed)
+
+        contentview = ContentView(self.iconview,
+                                  "document-open-recent-symbolic",
+                                  _("Select <b>New</b> to add a world clock"))
+        self.notebook.append_page(contentview, None)
+
+        self.storage = WorldClockStorage()
+        self.clocks = []
+        self.load_clocks()
+        self.show_all()
+
+        self.standalone = ClockStandalone()
+        self.notebook.append_page(self.standalone, None)
+
+        wallclock.connect("time-changed", self._update_clocks)
+
+    def _thumb_data_func(self, view, cell, store, i, data):
+        clock = store.get_value(i, 2)
+        cell.text = clock.time_string
+        cell.subtext = clock.day_string
+        if clock.is_light:
+            cell.props.pixbuf = self.daypixbuf
+            cell.css_class = "light"
+        else:
+            cell.props.pixbuf = self.nightpixbuf
+            cell.css_class = "dark"
+
+    def set_mode(self, mode):
+        self.mode = mode
+        if mode is Clock.Mode.NORMAL:
+            self.notebook.set_current_page(0)
+            self.iconview.set_selection_mode(False)
+        elif mode is Clock.Mode.STANDALONE:
+            self.notebook.set_current_page(1)
+        elif mode is Clock.Mode.SELECTION:
+            self.iconview.set_selection_mode(True)
+
+    def _update_clocks(self, *args):
+        for c in self.clocks:
+            c.update_time()
+        self.iconview.queue_draw()
+        self.standalone.update()
+        return True
+
+    def _on_item_activated(self, iconview, path):
+        clock = self.liststore[path][2]
+        self.standalone.set_clock(clock)
+        self.emit("item-activated")
+
+    def _on_selection_changed(self, iconview):
+        self.emit("selection-changed")
+
+    @GObject.Property(type=bool, default=False)
+    def can_select(self):
+        return len(self.liststore) != 0
+
+    def get_selection(self):
+        return self.iconview.get_selection()
+
+    def delete_selected(self):
+        selection = self.get_selection()
+        clocks = [self.liststore[path][2] for path in selection]
+        self.delete_clocks(clocks)
+        self.emit("selection-changed")
+
+    def load_clocks(self):
+        self.clocks = self.storage.load()
+        for clock in self.clocks:
+            self._add_clock_item(clock)
+
+    def add_clock(self, location):
+        if location.get_code() in [c.location.get_code() for c in self.clocks]:
+            # duplicate
+            return
+        clock = ClockItem(location)
+        self.clocks.append(clock)
+        self.storage.save(self.clocks)
+        self._add_clock_item(clock)
+        self.show_all()
+
+    def _add_clock_item(self, clock):
+        label = GLib.markup_escape_text(clock.name)
+        self.liststore.append([False, "<b>%s</b>" % label, clock])
+        self.notify("can-select")
+
+    def delete_clocks(self, clocks):
+        self.clocks = [c for c in self.clocks if c not in clocks]
+        self.storage.save(self.clocks)
+        self.liststore.clear()
+        self.load_clocks()
+        self.notify("can-select")
+
+    def open_new_dialog(self):
+        window = NewWorldClockDialog(self.get_toplevel())
+        window.connect("response", self._on_dialog_response)
+        window.show_all()
+
+    def _on_dialog_response(self, dialog, response):
+        if response == 1:
+            l = dialog.get_location()
+            self.add_clock(l)
+        dialog.destroy()
